@@ -44,15 +44,17 @@ from simpleio import map_range
 __version__ = "0.0.0-auto.0"
 __repo__ = "https://github.com/adafruit/Adafruit_CircuitPython_FONA.git"
 
-# pylint: disable=bad-whitespace
 FONA_DEFAULT_TIMEOUT_MS = 500  # TODO: Check this against arduino...
 
-# COMMANDS
+# Commands
 CMD_AT = b"AT"
-# REPLIES
+# Replies
 REPLY_OK = b"OK"
 REPLY_AT = b"AT"
 
+# Maximum number of fona800 and fona808 sockets
+FONA_MAX_SOCKETS = const(6)
+
 # FONA Versions
 FONA_800_L = const(0x01)
 FONA_800_H = const(0x6)
@@ -61,36 +63,35 @@ FONA_808_V2 = const(0x3)
 FONA_3G_A = const(0x4)
 FONA_3G_E = const(0x5)
 
-FONA_MAX_SOCKETS = const(6)
-
-FONA_SMS_STORAGE_SIM = b'"SM"'
-FONA_SMS_STORAGE_INTERNAL = b'"SM"'
-
-# pylint: enable=bad-whitespace
+# FONA preferred SMS storage
+FONA_SMS_STORAGE_SIM = b'"SM"'  # Storage on the SIM
+FONA_SMS_STORAGE_INTERNAL = b'"ME"'  # Internal storage on the FONA
 
 
 # pylint: disable=too-many-instance-attributes, too-many-public-methods
 class FONA:
     """CircuitPython FONA module interface.
     :param ~busio.uart UART: FONA UART connection.
-    :param ~digialio RST: FONA RST Pin.
+    :param ~digialio RST: FONA RST pin.
+    :param ~digialio RI: Optional FONA Ring Interrupt (RI) pin.
     :param bool debug: Enable debugging output.
 
     """
 
-    # Connection modes
-    TCP_MODE = const(0)
-    UDP_MODE = const(1)
+    TCP_MODE = const(0)  # TCP socket
+    UDP_MODE = const(1)  # UDP socket
 
     # pylint: disable=too-many-arguments
-    def __init__(self, uart, rst, debug=False):
+    def __init__(self, uart, rst, ri=None, debug=False):
         self._buf = b""  # shared buffer
         self._fona_type = 0
         self._debug = debug
 
         self._uart = uart
         self._rst = rst
-        self._rst.switch_to_output()
+        self._ri = ri
+        if self._ri is not None:
+            self._ri.switch_to_input()
         if not self._init_fona():
             raise RuntimeError("Unable to find FONA. Please check connections.")
 
@@ -101,15 +102,14 @@ class FONA:
 
         timeout = 7000
         while timeout > 0:
-            while self._uart.in_waiting:
-                if self._send_check_reply(CMD_AT, reply=REPLY_OK):
-                    break
+            if self._send_check_reply(CMD_AT, reply=REPLY_OK):
+                break
             if self._send_check_reply(CMD_AT, reply=REPLY_AT):
                 break
             time.sleep(0.5)
             timeout -= 500
 
-        if timeout <= 0:
+        if timeout <= 0:  # no response to AT, last ditch attempt
             self._send_check_reply(CMD_AT, reply=REPLY_OK)
             time.sleep(0.1)
             self._send_check_reply(CMD_AT, reply=REPLY_OK)
@@ -121,9 +121,9 @@ class FONA:
         self._send_check_reply(b"ATE0", reply=REPLY_OK)
         time.sleep(0.1)
 
+        self._read_line()
         if not self._send_check_reply(b"ATE0", reply=REPLY_OK):
             return False
-        time.sleep(0.1)
 
         # turn on hangupitude
         self._send_check_reply(b"AT+CVHU=0", reply=REPLY_OK)
@@ -132,8 +132,8 @@ class FONA:
         self._buf = b""
         self._uart.reset_input_buffer()
 
-        self.uart_write(b"ATI\r\n")
-        self.read_line(multiline=True)
+        self._uart_write(b"ATI\r\n")
+        self._read_line(multiline=True)
 
         if self._buf.find(b"SIM808 R14") != -1:
             self._fona_type = FONA_808_V2
@@ -146,8 +146,8 @@ class FONA:
 
         if self._fona_type == FONA_800_L:
             # determine if SIM800H
-            self.uart_write(b"AT+GMM\r\n")
-            self.read_line(multiline=True)
+            self._uart_write(b"AT+GMM\r\n")
+            self._read_line(multiline=True)
 
             if self._buf.find(b"SIM800H") != -1:
                 self._fona_type = FONA_800_H
@@ -155,19 +155,17 @@ class FONA:
 
     def factory_reset(self):
         """Resets modem to factory configuration."""
-        self.uart_write(b"ATZ\r\n")
+        self._uart_write(b"ATZ\r\n")
 
         if not self._expect_reply(REPLY_OK):
             return False
         return True
 
     def reset(self):
-        """Performs a hardware reset on the modem.
-        NOTE: This may take a few seconds to complete.
-
-        """
+        """Performs a hardware reset on the modem."""
         if self._debug:
             print("* Reset FONA")
+        self._rst.switch_to_output()
         self._rst.value = True
         time.sleep(0.01)
         self._rst.value = False
@@ -177,47 +175,47 @@ class FONA:
     @property
     # pylint: disable=too-many-return-statements
     def version(self):
-        """Returns FONA Version."""
+        """The version of the FONA module. Can be FONA_800_L,
+        FONA_800_H, FONA_808_V1, FONA_808_V2, FONA_3G_A, FONA3G_E.
+        """
         return self._fona_type
 
     @property
     def iemi(self):
-        """Returns FONA module's IEMI number."""
+        """FONA Module's IEMI (International Mobile Equipment Identity) number."""
         if self._debug:
             print("FONA IEMI")
         self._uart.reset_input_buffer()
 
-        self.uart_write(b"AT+GSN\r\n")
-        self.read_line(multiline=True)
+        self._uart_write(b"AT+GSN\r\n")
+        self._read_line(multiline=True)
         iemi = self._buf[0:15]
         return iemi.decode("utf-8")
 
     @property
     def local_ip(self):
-        """Returns the local IP Address, False if not set."""
-        self.uart_write(b"AT+CIFSR\r\n")
-        self.read_line()
+        """Module's local IP address, None if not set."""
+        self._uart_write(b"AT+CIFSR\r\n")
+        self._read_line()
         try:
             ip_addr = self.pretty_ip(self._buf)
         except ValueError:
-            return False
+            return None
         return ip_addr
 
     @property
     def iccid(self):
-        """Returns SIM Card's ICCID number as a string."""
+        """SIM Card's unique ICCID (Integrated Circuit Card Identifier)."""
         if self._debug:
             print("ICCID")
-        self.uart_write(b"AT+CCID\r\n")
-        self.read_line(timeout=2000)  # 6.2.23, 2sec max. response time
+        self._uart_write(b"AT+CCID\r\n")
+        self._read_line(timeout=2000)  # 6.2.23, 2sec max. response time
         iccid = self._buf.decode()
-        self.read_line()  # eat 'OK'
         return iccid
 
     @property
     def gprs(self):
-        """Returns module's GPRS state."""
-
+        """GPRS (General Packet Radio Services) power status."""
         if not self._send_parse_reply(b"AT+CGATT?", b"+CGATT: ", ":"):
             return False
         if not self._buf:
@@ -233,14 +231,10 @@ class FONA:
         if enable:
             apn_name, apn_user, apn_pass = apn
 
-            apn_name = apn_name.encode()
-            apn_user = apn_user.encode()
-            apn_pass = apn_pass.encode()
-
             # enable multi connection mode (3,1)
             if not self._send_check_reply(b"AT+CIPMUX=1", reply=REPLY_OK):
                 return False
-            self.read_line()
+            self._read_line()
 
             # enable receive data manually (7,2)
             if not self._send_check_reply(b"AT+CIPRXGET=1", reply=REPLY_OK):
@@ -264,33 +258,33 @@ class FONA:
             # Send command AT+SAPBR=3,1,"APN","<apn value>"
             # where <apn value> is the configured APN value.
             self._send_check_reply_quoted(
-                b'AT+SAPBR=3,1,"APN",', apn_name, REPLY_OK, 10000
+                b'AT+SAPBR=3,1,"APN",', apn_name.encode(), REPLY_OK, 10000
             )
 
             # send AT+CSTT,"apn","user","pass"
             self._uart.reset_input_buffer()
 
-            self.uart_write(b'AT+CSTT="' + apn_name)
+            self._uart_write(b'AT+CSTT="' + apn_name.encode())
 
             if apn_user is not None:
-                self.uart_write(b'","' + apn_user)
+                self._uart_write(b'","' + apn_user.encode())
 
             if apn_pass is not None:
-                self.uart_write(b'","' + apn_pass)
-            self.uart_write(b'"\r\n')
+                self._uart_write(b'","' + apn_pass.encode())
+            self._uart_write(b'"\r\n')
 
             if not self._get_reply(REPLY_OK):
                 return False
 
             # Set username
             if not self._send_check_reply_quoted(
-                b'AT+SAPBR=3,1,"USER",', apn_user, REPLY_OK, 10000
+                b'AT+SAPBR=3,1,"USER",', apn_user.encode(), REPLY_OK, 10000
             ):
                 return False
 
             # Set password
             if not self._send_check_reply_quoted(
-                b'AT+SAPBR=3,1,"PWD",', apn_pass, REPLY_OK, 100000
+                b'AT+SAPBR=3,1,"PWD",', apn_pass.encode(), REPLY_OK, 100000
             ):
                 return False
 
@@ -317,7 +311,8 @@ class FONA:
 
     @property
     def network_status(self):
-        """Returns cellular network status"""
+        """The status of the cellular network."""
+        self._read_line()
         if self._debug:
             print("Network status")
         if not self._send_parse_reply(b"AT+CREG?", b"+CREG: ", idx=1):
@@ -329,7 +324,9 @@ class FONA:
 
     @property
     def rssi(self):
-        """Returns cellular network's Received Signal Strength Indicator (RSSI)."""
+        """The received signal strength indicator for the cellular network
+        we are connected to.
+        """
         if self._debug:
             print("RSSI")
         if not self._send_parse_reply(b"AT+CSQ", b"+CSQ: "):
@@ -347,13 +344,12 @@ class FONA:
         if 2 <= reply_num <= 30:
             rssi = map_range(reply_num, 2, 30, -110, -54)
 
-        # read out the 'ok'
-        self.read_line()
+        self._read_line()  # eat the 'ok'
         return rssi
 
     @property
     def gps(self):
-        """Returns the GPS fix."""
+        """Module's GPS status."""
         if self._debug:
             print("GPS Fix")
         if self._fona_type == FONA_808_V2:
@@ -367,11 +363,7 @@ class FONA:
             status = int(self._buf[10:11].decode("utf-8"))
             if status == 1:
                 status = 3  # assume 3D fix
-            self.read_line()
-        elif self._fona_type == FONA_3G_A or self._fona_type == FONA_3G_E:
-            raise NotImplementedError(
-                "FONA 3G not currently supported by this library."
-            )
+            self._read_line()
         else:
             raise NotImplementedError(
                 "FONA 808 v1 not currently supported by this library."
@@ -380,10 +372,6 @@ class FONA:
 
     @gps.setter
     def gps(self, gps_on=False):
-        """Sets GPS module power, parses returned buffer.
-        :param bool gps_on: Enables the GPS module, disabled by default.
-
-        """
         if not (
             self._fona_type == FONA_3G_A
             or self._fona_type == FONA_3G_E
@@ -396,24 +384,22 @@ class FONA:
         if self._fona_type == FONA_808_V2:
             if not self._send_parse_reply(b"AT+CGPSPWR?", b"+CGPSPWR: ", ":"):
                 return False
-        self.read_line()
+        self._read_line()
         if not self._send_parse_reply(b"AT+CGNSPWR?", b"+CGNSPWR: ", ":"):
             return False
 
         state = self._buf
 
         if gps_on and not state:
-            self.read_line()
-            if self._fona_type == FONA_808_V2:
-                # try GNS
+            self._read_line()
+            if self._fona_type == FONA_808_V2:  # try GNS
                 if not self._send_check_reply(b"AT+CGNSPWR=1", reply=REPLY_OK):
                     return False
             else:
                 if not self._send_parse_reply(b"AT+CGPSPWR=1", reply_data=REPLY_OK):
                     return False
         else:
-            if self._fona_type == FONA_808_V2:
-                # try GNS
+            if self._fona_type == FONA_808_V2:  # try GNS
                 if not self._send_check_reply(b"AT+CGNSPWR=0", reply=REPLY_OK):
                     return False
                 if not self._send_check_reply(b"AT+CGPSPWR=0", reply=REPLY_OK):
@@ -421,29 +407,6 @@ class FONA:
 
         return True
 
-    def get_host_by_name(self, hostname):
-        """Converts a hostname to a packed 4-byte IP address.
-        Returns a 4 bytearray.
-        :param str hostname: Destination server.
-
-        """
-        if self._debug:
-            print("*** Get host by name")
-        if isinstance(hostname, str):
-            hostname = bytes(hostname, "utf-8")
-
-        if not self._send_check_reply(
-            b'AT+CDNSGIP="' + hostname + b'"\r\n', reply=REPLY_OK
-        ):
-            return False
-
-        # attempt to parse a response
-        self.read_line()
-        while not self._parse_reply(b"+CDNSGIP:", idx=2):
-            self.read_line()
-
-        return self._buf
-
     def pretty_ip(self, ip):  # pylint: disable=no-self-use, invalid-name
         """Converts a bytearray IP address to a dotted-quad string for printing"""
         return "%d.%d.%d.%d" % (ip[0], ip[1], ip[2], ip[3])
@@ -457,14 +420,13 @@ class FONA:
 
     @property
     def enable_sms_notification(self):
-        """Returns status of new SMS message notifications"""
+        """Checks if SMS notifications are enabled."""
         if not self._send_parse_reply(b"AT+CNMI?\r\n", b"+CNMI:", idx=1):
             return False
         return self._buf
 
     @enable_sms_notification.setter
     def enable_sms_notification(self, enable=True):
-        """Enables or disables new SMS message notifications."""
         if enable:
             if not self._send_check_reply(b"AT+CNMI=2,1\r\n", reply=REPLY_OK):
                 return False
@@ -473,6 +435,30 @@ class FONA:
                 return False
         return True
 
+    def receive_sms(self):
+        """Checks for a message notification from the FONA module,
+        replies back with the a tuple containing (sender, message).
+        NOTE: This method needs to be polled consistently due to the lack
+        of hw-based interrupts in CircuitPython.
+
+        """
+        if self._ri is not None:  # poll the RI pin
+            if self._ri.value:
+                return False, False
+        if not self._uart.in_waiting:  # otherwise, poll the UART
+            return False, False
+
+        self._read_line()  # parse the rcv'd URC
+        if not self._parse_reply(b"+CMTI: ", idx=1):
+            return False, False
+        slot = self._buf
+        sender, message = self.read_sms(slot)
+
+        if not self.delete_sms(slot):  # delete sms from module memory
+            return False, False
+
+        return sender, message.strip()
+
     def send_sms(self, phone_number, message):
         """Sends a message SMS to a phone number.
         :param int phone_number: Destination phone number.
@@ -486,41 +472,37 @@ class FONA:
         if not self._send_check_reply(b"AT+CMGF=1", reply=REPLY_OK):
             return False
 
-        self.uart_write(b'AT+CMGS="+' + str(phone_number).encode() + b'"' + b"\r")
-        self.read_line()
+        self._uart_write(b'AT+CMGS="+' + str(phone_number).encode() + b'"' + b"\r")
+        self._read_line()
 
-        # expect >
-        if self._buf[0] != 62:
+        if self._buf[0] != 62:  # expect '>'
             # promoting mark ('>') not found
             return False
-        self.read_line()
+        self._read_line()
 
         # write out message and ^z
-        self.uart_write((message + chr(26)).encode())
+        self._uart_write((message + chr(26)).encode())
 
         if self._fona_type == FONA_3G_A or self._fona_type == FONA_3G_E:
-            # eat 2x CRLF
-            self.read_line(200)
-            self.read_line(200)
-        # read +CMGS, wait ~10sec.
-        self.read_line(10000)
+            self._read_line(200)  # eat first 'CRLF'
+            self._read_line(200)  # eat second 'CRLF'
 
+        # read +CMGS, wait ~10sec.
+        self._read_line(10000)
         if not "+CMGS" in self._buf:
             return False
 
-        # read OK
         if not self._expect_reply(REPLY_OK):
             return False
         return True
 
-    @property
     def num_sms(self, sim_storage=True):
-        """Returns the number of SMS messages stored in memory, False if none stored.
-
+        """Returns the number of SMS messages stored in memory.
         :param bool sim_storage: SMS storage on the SIM, otherwise internal storage on FONA chip.
+
         """
         if not self._send_check_reply(b"AT+CMGF=1", reply=REPLY_OK):
-            return False
+            raise RuntimeError("Operating mode not supported by FONA module.")
 
         if sim_storage:  # ask how many SMS are stored
             if self._send_parse_reply(b"AT+CPMS?", FONA_SMS_STORAGE_SIM + b",", idx=1):
@@ -531,29 +513,17 @@ class FONA:
             ):
                 return self._buf
 
-        self.read_line()  # eat OK
+        self._read_line()  # eat OK
         if self._send_parse_reply(b"AT+CPMS?", b'"SM",', idx=1):
             return self._buf
 
-        self.read_line()  # eat OK
+        self._read_line()  # eat OK
         if self._send_parse_reply(b"AT+CPMS?", b'"SM_P",', idx=1):
             return self._buf
-        return False
-
-    def delete_all_sms(self):
-        """Deletes all SMS messages on the FONA SIM."""
-        self.read_line()
-        if not self._send_check_reply(b"AT+CMGF=1", reply=REPLY_OK):
-            return False
-
-        if not self._send_check_reply(
-            b'AT+CMGDA="DEL ALL"', reply=REPLY_OK, timeout=25000
-        ):
-            return False
-        return True
+        return 0
 
     def delete_sms(self, sms_slot):
-        """Deletes a SMS message in the provided SMS slot.
+        """Deletes a SMS message from a storage (internal or sim) slot
         :param int sms_slot: SMS SIM or FONA memory slot number.
 
         """
@@ -567,6 +537,24 @@ class FONA:
 
         return True
 
+    def delete_all_sms(self):
+        """Deletes all SMS messages on the FONA SIM."""
+        self._read_line()
+        if not self._send_check_reply(b"AT+CMGF=1", reply=REPLY_OK):
+            return False
+
+        if self._fona_type == FONA_3G_A or self._fona_type == FONA_3G_E:
+            num_sms = self.num_sms()
+            for slot in range(0, num_sms):
+                if not self.delete_sms(slot):
+                    return False
+        else:  # DEL ALL on 808
+            if not self._send_check_reply(
+                b'AT+CMGDA="DEL ALL"', reply=REPLY_OK, timeout=25000
+            ):
+                return False
+        return True
+
     def read_sms(self, sms_slot):
         """Reads and parses SMS messages from FONA device. Returns the SMS
         sender's phone number and the message contents as a tuple.
@@ -578,8 +566,8 @@ class FONA:
         if not self._send_check_reply(b"AT+CSDH=1", reply=REPLY_OK):
             return False
 
-        self.uart_write(b"AT+CMGR=" + str(sms_slot).encode() + b"\r\n")
-        self.read_line(1000)
+        self._uart_write(b"AT+CMGR=" + str(sms_slot).encode() + b"\r\n")
+        self._read_line(1000)
         resp = self._buf
 
         # get sender
@@ -597,39 +585,58 @@ class FONA:
         self._uart.readinto(self._buf)
         message = bytes(self._buf).decode()
         self._uart.reset_input_buffer()
-        self.read_line()  # eat 'OK'
+        self._read_line()  # eat 'OK'
 
         return sender, message
 
     ### Socket API (TCP, UDP) ###
 
-    def get_socket(self):
-        """Returns an avaliable socket (INITIAL or CLOSED state).
+    def get_host_by_name(self, hostname):
+        """Converts a hostname to a packed 4-byte IP address.
+        :param str hostname: Destination server.
 
         """
+        self._read_line()
+        if self._debug:
+            print("*** Get host by name")
+        if isinstance(hostname, str):
+            hostname = bytes(hostname, "utf-8")
+
+        if not self._send_check_reply(
+            b'AT+CDNSGIP="' + hostname + b'"\r\n', reply=REPLY_OK
+        ):
+            return False
+
+        self._read_line()
+        while not self._parse_reply(b"+CDNSGIP:", idx=2):
+            self._read_line()
+        return self._buf
+
+    def get_socket(self):
+        """Obtains a socket, if available."""
         if self._debug:
             print("*** Get socket")
 
-        self.uart_write(b"AT+CIPSTATUS\r\n")
-        self.read_line(100)  # OK
-        self.read_line(100)  # table header
+        self._uart_write(b"AT+CIPSTATUS\r\n")
+        self._read_line(100)  # OK
+        self._read_line(100)  # table header
 
         allocated_socket = 0
         for sock in range(0, FONA_MAX_SOCKETS):  # check if INITIAL state
-            self.read_line(100)
+            self._read_line(100)
             self._parse_reply(b"C:", idx=5)
             if self._buf.strip('"') == "INITIAL" or self._buf.strip('"') == "CLOSED":
                 allocated_socket = sock
                 break
         # read out the rest of the responses
         for _ in range(allocated_socket, FONA_MAX_SOCKETS):
-            self.read_line(100)
+            self._read_line(100)
         if self._debug:
             print("Allocated socket #%d" % allocated_socket)
         return allocated_socket
 
     def remote_ip(self, sock_num):
-        """Returns the IP address of the host who sent the current incoming packet.
+        """Returns the IP address of the remote server.
         :param int sock_num: Desired socket.
 
         """
@@ -637,14 +644,14 @@ class FONA:
             sock_num < FONA_MAX_SOCKETS
         ), "Provided socket exceeds the maximum number of \
                                              sockets for the FONA module."
-        self.uart_write(b"AT+CIPSTATUS=" + str(sock_num).encode() + b"\r\n")
-        self.read_line(100)
+        self._uart_write(b"AT+CIPSTATUS=" + str(sock_num).encode() + b"\r\n")
+        self._read_line(100)
 
         self._parse_reply(b"+CIPSTATUS:", idx=3)
         return self._buf
 
     def socket_status(self, sock_num):
-        """Returns if socket is connected.
+        """Returns the socket connection status, False if not connected.
         :param int sock_num: Desired socket number.
 
         """
@@ -654,10 +661,10 @@ class FONA:
                                              sockets for the FONA module."
         if not self._send_check_reply(b"AT+CIPSTATUS", reply=REPLY_OK, timeout=100):
             return False
-        self.read_line()
+        self._read_line()
 
         for state in range(0, sock_num + 1):  # read "C: <n>" for each active connection
-            self.read_line()
+            self._read_line()
             if state == sock_num:
                 break
         self._parse_reply(b"C:", idx=5)
@@ -666,7 +673,7 @@ class FONA:
 
         # eat the rest of the sockets
         for _ in range(sock_num, FONA_MAX_SOCKETS):
-            self.read_line()
+            self._read_line()
 
         if not "CONNECTED" in state:
             return False
@@ -674,7 +681,7 @@ class FONA:
         return True
 
     def socket_available(self, sock_num):
-        """Returns the amount of bytes to be read from the socket.
+        """Returns the amount of bytes available to be read from the socket.
         :param int sock_num: Desired socket to return bytes available from.
 
         """
@@ -691,8 +698,8 @@ class FONA:
         if self._debug:
             print("\t {} bytes available.".format(self._buf))
 
-        self.read_line()
-        self.read_line()
+        self._read_line()
+        self._read_line()
 
         return data
 
@@ -718,37 +725,28 @@ class FONA:
         ), "Provided socket exceeds the maximum number of \
                                              sockets for the FONA module."
 
-        if self._debug:
-            print(
-                "* FONA socket connect, socket={}, protocol={}, port={}, ip={}".format(
-                    sock_num, conn_mode, port, dest
-                )
-            )
-
         # Query local IP Address
-        self.uart_write(b"AT+CIFSR\r\n")
-        self.read_line()
+        self._uart_write(b"AT+CIFSR\r\n")
+        self._read_line()
 
         # Start connection
-        self.uart_write(b"AT+CIPSTART=" + str(sock_num).encode())
+        self._uart_write(b"AT+CIPSTART=" + str(sock_num).encode())
         if conn_mode == 0:
-            self.uart_write(b',"TCP","')
+            self._uart_write(b',"TCP","')
         else:
-            self.uart_write(b',"UDP","')
-        self.uart_write(dest.encode() + b'","' + str(port).encode() + b'"\r\n')
+            self._uart_write(b',"UDP","')
+        self._uart_write(dest.encode() + b'","' + str(port).encode() + b'"\r\n')
 
         if not self._expect_reply(REPLY_OK):
             return False
 
         if not self._expect_reply(b"CONNECT OK"):
             return False
-
         return True
 
-    def socket_close(self, sock_num, quick_close=1):
+    def socket_close(self, sock_num):
         """Close TCP or UDP connection
         :param int sock_num: Desired socket number.
-        :param int quick_close: Quickly or slowly close the socket. Enabled by default
 
         """
         if self._debug:
@@ -758,11 +756,15 @@ class FONA:
         ), "Provided socket exceeds the maximum number of \
                                              sockets for the FONA module."
 
-        self.uart_write(b"AT+CIPCLOSE=" + str(sock_num).encode() + b",")
-        self.uart_write(str(quick_close).encode() + b"\r\n")
-        self.read_line()
-        if not self._parse_reply(b"CLOSE OK", idx=0):
-            return False
+        self._uart_write(b"AT+CIPCLOSE=" + str(sock_num).encode() + b"\r\n")
+        self._read_line(3000)
+
+        if self._fona_type == FONA_3G_A or self._fona_type == FONA_3G_E:
+            if not self._expect_reply(REPLY_OK):
+                return False
+        else:
+            if not self._expect_reply(b"CLOSE OK"):
+                return False
         return True
 
     def socket_read(self, sock_num, length):
@@ -772,6 +774,7 @@ class FONA:
         :param int length: Desired length to read.
 
         """
+        self._read_line()
         assert (
             sock_num < FONA_MAX_SOCKETS
         ), "Provided socket exceeds the maximum number of \
@@ -779,43 +782,39 @@ class FONA:
         if self._debug:
             print("* socket read")
 
-        self.uart_write(b"AT+CIPRXGET=2,")
-        self.uart_write(str(sock_num).encode())
-        self.uart_write(b",")
-        self.uart_write(str(length).encode() + b"\r\n")
-
-        self.read_line()
+        self._uart_write(b"AT+CIPRXGET=2," + str(sock_num).encode() + b",")
+        self._uart_write(str(length).encode() + b"\r\n")
+        self._read_line()
 
         if not self._parse_reply(b"+CIPRXGET:"):
             return False
 
-        self._buf = self._uart.read(length)
+        return self._uart.read(length)
 
-        return self._buf
-
-    def socket_write(self, sock_num, buffer):
+    def socket_write(self, sock_num, buffer, timeout=3000):
         """Writes bytes to the socket.
         :param int sock_num: Desired socket number to write to.
         :param bytes buffer: Bytes to write to socket.
+        :param int timeout: Socket write timeout, in milliseconds.
 
         """
-        self.read_line()
+        self._read_line()
         assert (
             sock_num < FONA_MAX_SOCKETS
         ), "Provided socket exceeds the maximum number of \
                                              sockets for the FONA module."
 
         self._uart.reset_input_buffer()
-        self.uart_write(b"AT+CIPSEND=" + str(sock_num).encode())
-        self.uart_write(b"," + str(len(buffer)).encode() + b"\r\n")
-        self.read_line()
+        self._uart_write(b"AT+CIPSEND=" + str(sock_num).encode())
+        self._uart_write(b"," + str(len(buffer)).encode() + b"\r\n")
+        self._read_line()
 
         if self._buf[0] != 62:
             # promoting mark ('>') not found
             return False
 
-        self.uart_write(buffer + b"\r\n")
-        self.read_line(3000)
+        self._uart_write(buffer + b"\r\n")
+        self._read_line(timeout)
 
         if "SEND OK" not in self._buf.decode():
             return False
@@ -824,12 +823,7 @@ class FONA:
 
     ### UART Reply/Response Helpers ###
 
-    @property
-    def in_waiting(self):
-        """Returns number of bytes available to be read in the input buffer."""
-        return self._uart.in_waiting
-
-    def uart_write(self, buffer):
+    def _uart_write(self, buffer):
         """UART ``write`` with optional debug that prints
         the buffer before sending.
         :param bytes buffer: Buffer of bytes to send to the bus.
@@ -846,7 +840,7 @@ class FONA:
         :param str divider: Separator
 
         """
-        self.read_line()
+        self._read_line()
         self._get_reply(send_data)
 
         if not self._parse_reply(reply_data, divider, idx):
@@ -864,11 +858,11 @@ class FONA:
         self._uart.reset_input_buffer()
 
         if data is not None:
-            self.uart_write(data + b"\r\n")
+            self._uart_write(data + b"\r\n")
         else:
-            self.uart_write(prefix + suffix + b"\r\n")
+            self._uart_write(prefix + suffix + b"\r\n")
 
-        return self.read_line(timeout)
+        return self._read_line(timeout)
 
     def _parse_reply(self, reply, divider=",", idx=0):
         """Attempts to find reply in UART buffer, reads up to divider.
@@ -894,7 +888,7 @@ class FONA:
 
         return True
 
-    def read_line(self, timeout=FONA_DEFAULT_TIMEOUT_MS, multiline=False):
+    def _read_line(self, timeout=FONA_DEFAULT_TIMEOUT_MS, multiline=False):
         """Reads one or multiple lines into the buffer. Optionally prints the buffer
         after reading.
         :param int timeout: Time to wait for UART serial to reply, in seconds.
@@ -943,7 +937,7 @@ class FONA:
         :param bytes reply: Expected response from module.
 
         """
-        self.read_line()
+        self._read_line()
         if send is None:
             if not self._get_reply(prefix=prefix, suffix=suffix, timeout=timeout):
                 return False
@@ -984,16 +978,16 @@ class FONA:
         """
         self._uart.reset_input_buffer()
 
-        self.uart_write(prefix + b'"' + suffix + b'"\r\n')
+        self._uart_write(prefix + b'"' + suffix + b'"\r\n')
 
-        return self.read_line(timeout)
+        return self._read_line(timeout)
 
     def _expect_reply(self, reply, timeout=10000):
         """Reads line from FONA module and compares to reply from FONA module.
         :param bytes reply: Expected reply from module.
 
         """
-        self.read_line(timeout)
+        self._read_line(timeout)
         if reply not in self._buf:
             return False
         return True
